﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using Styx;
using Styx.Plugins;
using Styx.WoWInternals;
using Styx.WoWInternals.WoWObjects;
using System.Media;
using Styx.Helpers;
using System.IO;

namespace WatchTheChat
{
    public partial class Form1 : Form
    {
        private DateTime startedTime;

        public static LocalPlayer Me { get { return StyxWoW.Me; } }

        private string SoundPath = GlobalSettings.Instance.PluginsPath + @"\WatchTheChat\Sound\";

        public Form1()
        {
            InitializeComponent();

			//Link the log types to the rich text boxes in the form (For ease of use)
			Dictionary<string, RichTextBox> logwindows = new Dictionary<string, RichTextBox>()
			{
				{"bg", richTextBox1},
				{"whisper", richTextBox2},
				{"say", richTextBox3},
				{"guild", richTextBox4},
				{"log", richTextBox5}
			};

			//Initalize messages that we started!
			foreach (KeyValuePair<string, List<Message>> log in WatchTheChat.messages)
			{
				if (logwindows.ContainsKey(log.Key))
				{
					foreach (Message msg in log.Value)
					{
						logwindows[log.Key].Text += msg + Environment.NewLine;
					}
				}
			}
			
			//Load settings in WatchTheChat.Properties.Settings (Most are linked. TopMost affects the window)
			#region LoadSettings

			//TopMost
			this.TopMost = WatchTheChat.Settings.OnTop;
			AddMessage("log", "Loaded TopMost Settings");

			this.checkBoxBG.Checked = WatchTheChat.Settings.BG;
			this.checkBoxGuild.Checked = WatchTheChat.Settings.Guild;
			this.checkBoxSay.Checked = WatchTheChat.Settings.Say;
			this.checkBoxWhisper.Checked = WatchTheChat.Settings.Whisper;
			this.checkBoxPlayIfContains.Checked = WatchTheChat.Settings.AlertOnlyIfContains;
			this.checkBoxRunInBackground.Checked = WatchTheChat.Settings.RunInBackground;
			this.textBoxAliases.Text = WatchTheChat.Settings.Aliases;


			//Sound boxes
			string[] sounds = Directory.GetFiles(SoundPath, @"*.wav");

			foreach (string soundFile in sounds)
			{
				comboBoxAlert.Items.Add(Path.GetFileNameWithoutExtension(soundFile));
				comboBoxSiren.Items.Add(Path.GetFileNameWithoutExtension(soundFile));
			}

			try
			{
				comboBoxAlert.SelectedIndex = comboBoxAlert.Items.IndexOf(WatchTheChat.Settings.AlertSound);
			}
			catch { }

			try
			{
				comboBoxSiren.SelectedIndex = comboBoxSiren.Items.IndexOf(WatchTheChat.Settings.SirenSound);
			}
			catch { }

			#endregion

            //Setting the window title
            this.Text = "WatchTheChat - " + Me.Name;

			//Add events (Doing it here, so settings populate, then events get fired when changed)
			this.comboBoxSiren.SelectedIndexChanged += new System.EventHandler(this.formObject_Changed);
			this.comboBoxAlert.SelectedIndexChanged += new System.EventHandler(this.formObject_Changed);
			this.textBoxAliases.TextChanged += new System.EventHandler(this.formObject_Changed);
			this.checkBoxRunInBackground.CheckedChanged += new System.EventHandler(this.formObject_Changed);
			this.checkBoxOnTop.CheckedChanged += new System.EventHandler(this.checkBoxOnTop_CheckedChanged);
			this.checkBoxGuild.CheckedChanged += new System.EventHandler(this.formObject_Changed);
			this.checkBoxPlayIfContains.CheckedChanged += new System.EventHandler(this.formObject_Changed);
			this.checkBoxBG.CheckedChanged += new System.EventHandler(this.formObject_Changed);
			this.checkBoxSay.CheckedChanged += new System.EventHandler(this.formObject_Changed);
			this.checkBoxWhisper.CheckedChanged += new System.EventHandler(this.formObject_Changed);

			//We added a bunch of log entries when the form was initialized. Update the labels.
			UpdateStatusLabel();

			//Writes that the initialazion is completed to the log window
			AddMessage("log", "Watch The Chat (v2), form initialized!");
       }

		public void AddMessage(string msgType, Message message)
		{
			Dictionary<string, RichTextBox> logwindows = new Dictionary<string, RichTextBox>()
			{
				{"bg", richTextBox1},
				{"whisper", richTextBox2},
				{"say", richTextBox3},
				{"guild", richTextBox4},
				{"log", richTextBox5}
			};

			if (logwindows.ContainsKey(msgType))
			{
				logwindows[msgType].Text += message + Environment.NewLine;
				logwindows[msgType].SelectionStart = logwindows[msgType].TextLength;
				logwindows[msgType].ScrollToCaret();
			}
		}

        public void UpdateStatusLabel()
        {
			label1.Text = String.Format("Whispers: {0} Say: {1} ({2}) Guild: {3} ({4}) BG: {5} ({6})",
				WatchTheChat.messages["whisper"].Count,
				WatchTheChat.messages["say"].Count,
				WatchTheChat.messages["say"].Count(m => m.ContainsMyName),
				WatchTheChat.messages["guild"].Count,
				WatchTheChat.messages["guild"].Count(m => m.ContainsMyName),
				WatchTheChat.messages["bg"].Count,
				WatchTheChat.messages["bg"].Count(m => m.ContainsMyName)
			);
        }

        public void SaveToTextFile()
        {
            //Set the time when saved
            startedTime = DateTime.Now;

            #region FileSave
            //Save the file
            if (!Directory.Exists(GlobalSettings.Instance.PluginsPath + @"\WatchTheChat\ChatLogs"))
            {
                Directory.CreateDirectory(GlobalSettings.Instance.PluginsPath + @"\Plugins\WatchTheChat\ChatLogs");
            }

            StreamWriter writer = new StreamWriter(GlobalSettings.Instance.PluginsPath + @"\Plugins\WatchTheChat\ChatLogs\" + startedTime.ToLongTimeString().Replace(':', '-') + ".txt");

            //Battleground
            writer.WriteLine(":::Battleground:::");
            writer.WriteLine("");

            foreach (string str in richTextBox1.Lines)
            {
                writer.WriteLine(str);
            }

            //WHISPERS
            writer.WriteLine(":::Whispers:::");
            writer.WriteLine("");

            foreach (string str in richTextBox2.Lines)
            {
                writer.WriteLine(str);
            }

            //SAY
            writer.WriteLine(":::SAY:::");
            writer.WriteLine("");

            foreach (string str in richTextBox4.Lines)
            {
                writer.WriteLine(str);
            }

            //GUILD
            writer.WriteLine(":::GUILD:::");
            writer.WriteLine("");

            foreach (string str in richTextBox5.Lines)
            {
                writer.WriteLine(str);
            }

            writer.Close();
            richTextBox3.Text += "File saved successfully" + Environment.NewLine;
            #endregion
        }

        private void button1_Click(object sender, EventArgs e)
        {
            SaveToTextFile();
            startedTime = DateTime.Now;
            richTextBox3.Text += "Changed save time" + Environment.NewLine;
        }

		private void Form1_FormClosing(object sender, FormClosingEventArgs e)
		{
			//Hide the form instead of closing it
			this.Hide();
			e.Cancel = true;
		}

		private void SaveSettings()
		{
			WatchTheChat.Settings.BG = this.checkBoxBG.Checked;
			WatchTheChat.Settings.Whisper = this.checkBoxWhisper.Checked;
			WatchTheChat.Settings.Say = this.checkBoxSay.Checked;
			WatchTheChat.Settings.Guild = this.checkBoxGuild.Checked;
			WatchTheChat.Settings.AlertOnlyIfContains = this.checkBoxPlayIfContains.Checked;
			WatchTheChat.Settings.OnTop = this.checkBoxOnTop.Checked;
			WatchTheChat.Settings.RunInBackground = this.checkBoxRunInBackground.Checked;
			WatchTheChat.Settings.Aliases = this.textBoxAliases.Text;
			WatchTheChat.Settings.AlertSound = (string)this.comboBoxAlert.SelectedItem;
			WatchTheChat.Settings.SirenSound = (string)this.comboBoxSiren.SelectedItem;
			WatchTheChat.Settings.Save();
		}

		private void formObject_Changed(object sender, EventArgs e)
		{
			SaveSettings();
			AddMessage("log", "Settings updated and saved");
		}

		private void checkBoxOnTop_CheckedChanged(object sender, EventArgs e)
		{
			formObject_Changed(sender, e);
			this.TopMost = WatchTheChat.Settings.OnTop;
		}

		private void PreviewSound(string soundFile)
		{
			if (soundFile != null)
			{
				if (File.Exists(SoundPath + soundFile + @".wav"))
				{
					new SoundPlayer(SoundPath + soundFile + @".wav").Play();
				}
				else
				{
					MessageBox.Show("Unable to find sound file", "Error", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
				}
			}
		}
		private void btnPreviewAlert_Click(object sender, EventArgs e)
		{
			PreviewSound((string)comboBoxAlert.SelectedItem);
		}

		private void btnPreviewSiren_Click(object sender, EventArgs e)
		{
			PreviewSound((string)comboBoxSiren.SelectedItem);
		}
    }
}