﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Styx;
using Styx.WoWInternals.WoWObjects;
using Styx.WoWInternals;
using System.Windows.Forms;
using Styx.Helpers;
using System.Media;
using Styx.Plugins;
using Styx.Common;
using Styx.CommonBot;
using System.Windows.Media;

namespace WatchTheChat
{
	enum AlertSounds
	{
		Siren,
		Alert
	}

	public class Message
	{
		private string _sender = null;
		private string _message = null;
		private DateTime _datetime = DateTime.Now;

		public static implicit operator Message(string Message)
		{
			return new Message(Message);
		}

		public Message() : this(null, null) { }

		public Message(string Message) : this(null, Message) { }

		public Message(string Sender, string Message)
		{
			if ( Sender != null )
				_sender = Sender.Trim();
			
			if ( Message != null )
				_message = Message.Trim();
		}

		public string Sender
		{
			get { return _sender; }
		}

		public string Msg
		{
			get { return _message; }
		}

		public override string ToString()
		{
			if (_sender == null)
			{
				return String.Format("{0}: {1}", _datetime.ToString("h:mm tt"), _message);
			}
			else
			{
				return String.Format("{0}: [{1}] - {2}", _datetime.ToString("h:mm tt"), _sender, _message);
			}
		}

		public bool ContainsMyName
		{
			get
			{
				if (_message.ToLower().Contains(StyxWoW.Me.Name.ToLower()))
					return true;

				if (WatchTheChat.Settings.Aliases.Trim() != "")
				{
					string[] split = WatchTheChat.Settings.Aliases.Split(',');

					foreach (string alias in split)
					{
						if (alias.Trim() != "" && _message.ToLower().Contains(alias.Trim().ToLower()))
							return true;
					}
				}

				return false;
			}
		}
	}

    class WatchTheChat : HBPlugin
    {
        public override string Author { get { return "Zapman (v2 by Danbopes)"; } }
        public override Version Version { get { return new Version(2, 1, 0, 0); } }
        public override string Name { get { return "WatchTheChat"; } }
        public static LocalPlayer Me { get { return StyxWoW.Me; } }
        public override bool WantButton { get { return true; } }
        public override string ButtonText { get { return "Show"; } }
		public static WatchTheChatSettings Settings = new WatchTheChatSettings();

        public bool hasBeenInitialized = false;
        public Form1 myForm = null;

		public static Dictionary<string, List<Message>> messages = new Dictionary<string, List<Message>>()
			{
				{"bg", new List<Message>()},
				{"say", new List<Message>()},
				{"guild", new List<Message>()},
				{"whisper", new List<Message>()},
				{"log", new List<Message>()}
			};

		private string avoidSpamBG = "", avoidSpamWhisper = "", avoidSpamSay = "",
			avoidSpamGuild = "";

		public void Log(string message, params object[] objects)
		{
			Logging.Write(
                Colors.DodgerBlue,
                String.Format("[WatchTheChat v{0}] {1}", Version.Major, message),
				objects
			);
		}
		public override void Initialize()
		{
			if (!hasBeenInitialized)
			{
				hasBeenInitialized = true;

				Settings.Load();

				if (Settings.RunInBackground)
				{
					AttachEvents();
				}

				Log("Loaded!");
			}
		}

		private void AttachEvents()
		{
			//Battleground
			Chat.Battleground += newAllianceBattleground;
			//richTextBox3.Text += "Battleground Initialized!" + Environment.NewLine;

			//Whisper
            Chat.Whisper += newWhisper;
			//richTextBox3.Text += "Whisper Initialized!" + Environment.NewLine;

			//Say
            Chat.Say += newSay;
			//richTextBox3.Text += "Say Initialized!" + Environment.NewLine;

			//Guild
            Chat.Guild += newGuild;
			//richTextBox3.Text += "Guild Initialized!" + Environment.NewLine;

			Lua.Events.DetachEvent("CHAT_MSG_BN_WHISPER", newBNWhisper);
			Lua.Events.AttachEvent("CHAT_MSG_BN_WHISPER", newBNWhisper);
			Log("Attached Chat Events!");
		}

		public override void Dispose()
		{
			//Battleground
            Chat.Battleground -= newAllianceBattleground;

			//Whisper
            Chat.Whisper -= newWhisper;

			//Say
            Chat.Say -= newSay;

			//Guild
            Chat.Guild -= newGuild;

			Lua.Events.DetachEvent("CHAT_MSG_BN_WHISPER", newBNWhisper);

			if (myForm != null)
			{
				myForm.Dispose();
			}

			base.Dispose();
		}

		public override void Pulse() {}

		private void NewMessage(string msgType, Message msg)
		{
			//Logging.Write(System.Drawing.Color.DodgerBlue, "AddMessage");
			messages[msgType].Add(msg);

			if ( myForm != null )
			{
				myForm.UpdateStatusLabel();
				myForm.AddMessage(msgType, msg);
			}

			if (msg.Sender != null)
			{
				Log("[{0}] {1}", msgType.ToUpper(), msg);
			}
		}

		public void newBNWhisper(object sender, LuaEventArgs args)
		{
			string message = args.Args[0].ToString();
			string presenceId = args.Args[12].ToString();
			string senderName = Lua.GetReturnValues(String.Format("return BNGetFriendInfoByID({0})", presenceId))[3];

			if (avoidSpamWhisper != message)
			{
				//Writes the whisper to the whisper window
				avoidSpamWhisper = message;

				//Will check if we are going to play alert sound
				if (Settings.Whisper)
				{
					//Will alert the user
					alert(AlertSounds.Siren);
				}

				NewMessage("whisper", new Message(senderName, message));
			}
		}

        public void newGuild(Chat.ChatGuildEventArgs arg)
		{
			if (avoidSpamGuild != arg.Message)
			{
				//Store last message in avoidSpamGuild
				avoidSpamGuild = arg.Message;

				Message msg = new Message(arg.Author, arg.Message);

				if (Settings.Guild)
				{
					if (Settings.AlertOnlyIfContains)
					{
						if (msg.ContainsMyName)
						{
							alert(AlertSounds.Siren);
						}
					}
					else 
					{
						if (msg.ContainsMyName)
						{
							//Lets alert with siren1 as it contains my name!
							alert(AlertSounds.Siren);
						}
						else
						{
							//Lets do a normal alert
							alert(AlertSounds.Alert);
						}
					}
				}

				NewMessage("guild", msg);

				//Writes the guild say to the guild window
				//richTextBox5.Text += "[" + arg.Author + "] - " + arg.Message + Environment.NewLine;
			}
		}

        public void newSay(Chat.ChatAuthoredEventArgs arg)
		{
			if (avoidSpamSay != arg.Message)
			{
				//Store last message in avoidSpamSay
				avoidSpamSay = arg.Message;

				Message msg = new Message(arg.Author, arg.Message);

				if (Settings.Say)
				{
					if (Settings.AlertOnlyIfContains)
					{
						if (msg.ContainsMyName)
						{
							alert(AlertSounds.Siren);
						}
					}
					else
					{
						if (msg.ContainsMyName)
						{
							//Lets alert with siren as it contains my name!
							alert(AlertSounds.Siren);
						}
						else
						{
							//Lets do a normal alert
							alert(AlertSounds.Alert);
						}
					}
				}

				NewMessage("say", msg);

				//Writes the guild say to the guild window
				//richTextBox5.Text += "[" + arg.Author + "] - " + arg.Message + Environment.NewLine;
			}
		}

        public void newAllianceBattleground(Chat.ChatAuthoredEventArgs arg)
		{
			if (avoidSpamBG != arg.Message)
			{
				//Store last message in avoidSpamBG
				avoidSpamBG = arg.Message;

				Message msg = new Message(arg.Author, arg.Message);

				if (Settings.BG)
				{
					if (Settings.AlertOnlyIfContains)
					{
						if (msg.ContainsMyName)
						{
							alert(AlertSounds.Siren);
						}
					}
					else
					{
						if (msg.ContainsMyName)
						{
							//Lets alert with siren as it contains my name!
							alert(AlertSounds.Siren);
						}
						else
						{
							//Lets do a normal alert
							alert(AlertSounds.Alert);
						}
					}
				}

				NewMessage("bg", msg);

				//Writes the guild say to the guild window
				//richTextBox5.Text += "[" + arg.Author + "] - " + arg.Message + Environment.NewLine;
			}
		}

        public void newWhisper(Chat.ChatAuthoredEventArgs arg)
		{
			if (avoidSpamWhisper != arg.Message)
			{
				//Writes the whisper to the whisper window
				avoidSpamWhisper = arg.Message;

				//Will check if we are going to play alert sound
				if (Settings.Whisper)
				{
					//Will alert the user
					alert(AlertSounds.Siren);
				}

				NewMessage("whisper", new Message(arg.Author, arg.Message));
			}
		}

		public void alert(AlertSounds alert)
		{
			//Play sound!
			//richTextBox3.Text += "Playing sound (" + sound + ")" + Environment.NewLine;
			string sound;
			if (alert == AlertSounds.Alert)
			{
				sound = Settings.AlertSound + ".wav";
			}
			else
			{
				sound = Settings.SirenSound + ".wav";
			}
			NewMessage("log", "Playing sound (" + sound + ")");
			new SoundPlayer(GlobalSettings.Instance.PluginsPath + @"\WatchTheChat\Sound\" + sound).Play();
		}

        public override void OnButtonPress()
        {
            //Avoid multiple forms at once
			if (myForm == null)
			{
				//Don't have run in background enabled? Start it now that they are loading the form.
				if ( !Settings.RunInBackground )
					AttachEvents();

				myForm = new Form1();
				myForm.Show();
			}
			else if (!myForm.Visible)
			{
				myForm.Show();
				myForm.Focus();
			}
			else
			{
				myForm.Focus();
			}
        }
	}
}
